package me.mrletsplay.videobase.library;

import java.util.HashMap;
import java.util.Map;

import me.mrletsplay.mrcore.json.JSONObject;
import me.mrletsplay.mrcore.json.converter.JSONConstructor;
import me.mrletsplay.mrcore.json.converter.JSONConverter;
import me.mrletsplay.mrcore.json.converter.JSONConvertible;
import me.mrletsplay.mrcore.json.converter.JSONValue;

public class LibraryMetadata implements JSONConvertible {

	@JSONValue("default")
	private VideoMetadata defaultMetadata;

	private Map<String, VideoMetadata> overrides;

	@JSONConstructor
	public LibraryMetadata() {
		this.overrides = new HashMap<>();
	}

	public VideoMetadata getDefault() {
		return defaultMetadata;
	}

	public Map<String, VideoMetadata> getOverrides() {
		return overrides;
	}

	@Override
	public void preDeserialize(JSONObject object) {
		JSONObject ov = object.optJSONObject("overrides").orElse(null);
		if(ov != null) {
			for(String key : ov.keys()) {
				overrides.put(key, JSONConverter.decodeObject(ov.getJSONObject(key), VideoMetadata.class));
			}
		}
	}

	@Override
	public void preSerialize(JSONObject object) {
		JSONObject ov = new JSONObject();
		for(String key : overrides.keySet()) {
			ov.put(key, overrides.get(key).toJSON());
		}
		object.put("overrides", ov);
	}

}
