package me.mrletsplay.videobase.rest;

import java.util.List;

import me.mrletsplay.mrcore.json.JSONArray;
import me.mrletsplay.mrcore.json.JSONObject;
import me.mrletsplay.simplehttpserver.http.HttpRequestMethod;
import me.mrletsplay.simplehttpserver.http.HttpStatusCodes;
import me.mrletsplay.simplehttpserver.http.endpoint.Endpoint;
import me.mrletsplay.simplehttpserver.http.endpoint.EndpointCollection;
import me.mrletsplay.simplehttpserver.http.endpoint.RequestParameter;
import me.mrletsplay.simplehttpserver.http.header.DefaultClientContentTypes;
import me.mrletsplay.simplehttpserver.http.request.HttpRequestContext;
import me.mrletsplay.simplehttpserver.http.response.JsonResponse;
import me.mrletsplay.simplehttpserver.http.response.TextResponse;
import me.mrletsplay.simplehttpserver.http.validation.result.ValidationResult;
import me.mrletsplay.videobase.VideoBase;
import me.mrletsplay.videobase.library.Library;
import me.mrletsplay.videobase.library.Video;
import me.mrletsplay.videobase.library.VideoMetadata;

public class LibraryAPI implements EndpointCollection {

	@Endpoint(method = HttpRequestMethod.GET, path = "")
	public void getLibrary(HttpRequestContext ctx) {
		String groupBy = ctx.getRequestedPath().getQuery().getFirst("groupBy", "none").toLowerCase();

		Library library = VideoBase.getLibrary();

		JSONObject libraryObj = new JSONObject();

		JSONObject videosObj = new JSONObject();

		List<String> groups;
		switch(groupBy) {
			case "series": {
				groups = library.getSeries();
				break;
			}
			case "author": {
				groups = library.getAuthors();
				break;
			}
			case "none":
			default:
				groups = List.of("all");
				break;
		}

		for(String group : groups) {
			List<Video> videos;
			switch(groupBy) {
				case "series": {
					videos = library.findVideosBy(group, null);
					break;
				}
				case "author": {
					videos = library.findVideosBy(null, group);
					break;
				}
				case "none":
				default:
					videos = library.getVideos();
					break;
			}

			JSONArray videosArr = new JSONArray();
			for(Video video : videos) {
				videosArr.add(videoToJSON(video));
			}
			videosObj.put(group, videosArr);
		}

		if(!"none".equals("groupBy")) {

		}

		libraryObj.put("videos", videosObj);

		ctx.respond(HttpStatusCodes.OK_200, new JsonResponse(libraryObj));
	}

	@Endpoint(method = HttpRequestMethod.GET, path = "/video/{video}", pathPattern = true)
	public void getVideo(HttpRequestContext ctx, @RequestParameter("video") String videoId) {
		Video video = VideoBase.getLibrary().findVideoById(videoId);
		if(video == null) {
			ctx.respond(HttpStatusCodes.NOT_FOUND_404, new TextResponse("Not found"));
			return;
		}

		ctx.respond(HttpStatusCodes.OK_200, new JsonResponse(videoToJSON(video)));
	}

	@Endpoint(method = HttpRequestMethod.PUT, path = "/video/{video}/metadata", pathPattern = true)
	public void updateVideoMetadata(HttpRequestContext ctx, @RequestParameter("video") String videoId) {
		Library library = VideoBase.getLibrary();
		Video video = VideoBase.getLibrary().findVideoById(videoId);
		if(video == null) {
			ctx.respond(HttpStatusCodes.NOT_FOUND_404, new TextResponse("Not found"));
			return;
		}

		JSONObject metadata = ctx.expectContent(DefaultClientContentTypes.JSON_OBJECT);
		if(metadata == null) {
			ctx.respond(HttpStatusCodes.BAD_REQUEST_400, ValidationResult.error("_", "Invalid content").asJsonResponse());
			return;
		}

		ValidationResult result = VideoMetadata.VALIDATOR.validate(metadata);
		if(!result.isOk()) {
			ctx.respond(HttpStatusCodes.BAD_REQUEST_400, result.asJsonResponse());
			return;
		}

		library.updateMetadata(video, new VideoMetadata(metadata));
		ctx.respond(HttpStatusCodes.OK_200, new JsonResponse(video.getMetadata().getRaw()));
	}

	private JSONObject videoToJSON(Video video) {
		JSONObject v = new JSONObject();
		v.put("id", video.getId());
		v.put("metadata", video.getMetadata().getRaw());
		return v;
	}

	@Override
	public String getBasePath() {
		return "/api/library";
	}

}
